local MAJOR, MINOR = "Vengeance", 1;
local Addon = LibStub("AceAddon-3.0"):NewAddon(MAJOR
	,"AceEvent-3.0"
);
-- debug
_G.Vengeance = Addon;

VengeanceSavedVariable = {
	FrameRect = {},
	ShowAlways = true,
	MouseEnabled = true,
	Opacity = 1,
	Texture = nil,
	ColorBar = nil,
	ColorBackground = nil,
	ShowBorder = true,
	BorderWidth = 3,
};

local floor = math.floor;
local min = math.min;
local VENGEANCE_SPELL_ID = 93099;

local function ToKiloNumber(number)
	return (floor(number/100+0.5)/10).."k";
end
local function ToMegaNumber(number)
	return (floor(number/100000+0.5)/10).."M";
end
local function ToPercentNumber(frac)
	return (floor(frac*1000+.5)/10).."%";
end

function Addon:OnEnable()
	-- Get spell name:
	-- Vengeance spell id's: 93099 (DK), 84840 (Druid), 84839 (Paladin), 93098 (Warrior) 
	-- All have the same name though, so we only need one.
	self.VengeanceName = GetSpellInfo(VENGEANCE_SPELL_ID);
	self.BaseHealth = 0;
	self.Stamina = 0;

	local SV = VengeanceSavedVariable;
	local SM = LibStub("LibSharedMedia-3.0");
	local Frame = CreateFrame("StatusBar");
	Frame:SetParent(UIParent);
	Frame:SetPoint("CENTER");
	Frame:SetWidth(SV.Width or 200);
	Frame:SetHeight(SV.Height or 20);
	Frame:SetAlpha(SV.Opacity or 1);
	Frame:SetOrientation("HORIZONTAL");
	Frame:SetMovable(true);
	Frame:SetMinMaxValues(0, 100);
	Frame:SetValue(0);
	self.Frame = Frame;
	
	local Border = Frame:CreateTexture(nil, "BACKGROUND");
	Border:SetTexture(0, 0, 0, .3);
	Border:SetPoint("TOPLEFT", Frame, -(SV.BorderWidth or 3), SV.BorderWidth or 3);
	Border:SetPoint("BOTTOMRIGHT", Frame, SV.BorderWidth or 3, -(SV.BorderWidth or 3));
	if not SV.ShowBorder then
		Border:Hide();
	end

	local BG = Frame:CreateTexture(nil, "BORDER");
	BG:SetTexture(0, 0, 0, .5);
	BG:SetAllPoints(Frame);

	local FG = Frame:CreateTexture(nil, "ARTWORK");
	FG:SetTexture(.8, .6, .1, 1);
	Frame:SetStatusBarTexture(FG);
	
	local TX = Frame:CreateFontString(nil, "OVERLAY");
	TX:SetFontObject(GameFontNormal);
	TX:SetTextColor(.5, .5, .5, 1);
	TX:SetAllPoints(Frame);
	TX:SetJustifyH("CENTER");
	TX:SetJustifyV("MIDDLE");
	
	self.SetTexture = function(self, tx)
		if tx then
			FG:SetTexture(SM:Fetch("statusbar", tx));
		elseif SV.ColorBar then
			FG:SetTexture(unpack(SV.ColorBar));
		end
		SV.Texture = tx;
		Frame:SetStatusBarTexture(FG);
	end
	self.SetColorBar = function(self, r,g,b,a)
		if SV.Texture then
			FG:SetVertexColor(r, g, b, a);
		else
			FG:SetTexture(r, g, b, a);
		end
		Frame:SetStatusBarTexture(FG);
		SV.ColorBar = {r,g,b,a};
	end
	self.SetColorBackground = function(self, r,g,b,a)
		BG:SetTexture(r, g, b, a);
		SV.ColorBackground = {r,g,b,a};
	end
	self.SetColorText = function(self, r,g,b,a)
		TX:SetTextColor(r, g, b, a);
		SV.ColorText = {r,g,b,a};
	end
	self.SetBorderVisibility = function(self, val)
		if val then
			Border:Show();
		else
			Border:Hide();
		end
		SV.ShowBorder = val;
	end
	self.SetBorderWidth = function(self, val)
		Border:ClearAllPoints();
		Border:SetPoint("TOPLEFT", Frame, -val, val);
		Border:SetPoint("BOTTOMRIGHT", Frame, val, -val);
	end
	self.OnUpdate = function(self)
		local Exists = select(14, UnitBuff("player", self.VengeanceName));
		local v = Exists or 0;
		-- select(14, UnitBuff("player", "Vengeance")) or 0

		if v == 0 then
			if Frame:IsShown() and not SV.ShowAlways then
				self:UnregisterEvent("UNIT_MAXHEALTH");
				self:UnregisterEvent("PLAYER_LEVEL_UP");
				self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
				self:UnregisterEvent("PLAYER_TALENT_UPDATE");
				Frame:Hide();
			end
		elseif not Frame:IsShown() then
			Frame:Show();
			self:RegisterEvent("UNIT_AURA", "OnUpdate");
			self:RegisterEvent("UNIT_MAXHEALTH", "OnMaxHealthUpdate");
			self:RegisterEvent("PLAYER_LEVEL_UP", "OnMaxHealthUpdate");
			self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "OnMaxHealthUpdate");
			self:RegisterEvent("PLAYER_TALENT_UPDATE", "OnMaxHealthUpdate");
			self:CalculateVengeanceMax();
		end

		Frame:SetValue(v);
		TX:SetText(v.." - "..ToPercentNumber(v/self.VengeanceMax));
		self.VengeanceValue = v;
	end;
	Frame:SetScript("OnMouseDown", function(self)
		if IsAltKeyDown() then
			self:StartMoving();
		end
	end);
	Frame:SetScript("OnMouseUp", function(self)
		self:StopMovingOrSizing();
		SV.FrameRect = {self:GetRect()};
	end);
	if #SV.FrameRect == 4 then
		local p = SV.FrameRect;
		Frame:SetPoint("BOTTOMLEFT", p[1], p[2])
		Frame:SetWidth(p[3]);
		Frame:SetHeight(p[4]);
	end
	if not SV.ShowAlways then
		Frame:Hide();
	else
		Frame:Show();
	end
	Frame:EnableMouse(SV.MouseEnabled);

	if VengeanceSavedVariable.ShowAlways then
		self:Show();
	else
		self:RegisterEvent("UNIT_AURA", "OnUpdate");
	end
	
	self:SetTexture(SV.Texture);
	self:SetColorBar(unpack(SV.ColorBar or {1, 1, 0, 1}));
	self:SetColorText(unpack(SV.ColorText or {.5, .5, .5, 1}));
	self:SetColorBackground(unpack(SV.ColorBackground or {0, 0, 0, 0.5}));
end

function Addon:OnMaxHealthUpdate(e)
	self:CalculateVengeanceMax();
	self:OnUpdate();
end

function Addon:CalculateVengeanceMax()
	local _, stamina = UnitStat("player", 3);
	self.Stamina = stamina;
	local basehp = (UnitHealthMax("player") or 0) - stamina*(UnitHPPerStamina("player") or 10);
	--print(MAJOR, "basehp", basehp, "stamina", stamina, "BaseHealth", self.BaseHealth);
	if self.BaseHealth > 0 then
		self.BaseHealth = min(self.BaseHealth, basehp);
	else
		self.BaseHealth = basehp;
	end
	--print(MAJOR, "BaseHealth", self.BaseHealth);
	self.VengeanceMax = self.BaseHealth * .1 + stamina;
	self.Frame:SetMinMaxValues(0, self.VengeanceMax);
end

function Addon:AurasChanged(e, ...)
	local s = strjoin(" ", e, tostringall(...));
	DEFAULT_CHAT_FRAME:AddMessage(MAJOR..": "..s);
end

function Addon:Show()
	self.Frame:Show();
	self:RegisterEvent("UNIT_AURA", "OnUpdate");
	self:RegisterEvent("UNIT_MAXHEALTH", "OnMaxHealthUpdate");
	self:RegisterEvent("PLAYER_LEVEL_UP", "OnMaxHealthUpdate");
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "OnMaxHealthUpdate");
	self:RegisterEvent("PLAYER_TALENT_UPDATE", "OnMaxHealthUpdate");
	
	self:OnMaxHealthUpdate();
end

function Addon:Hide()
	self:UnregisterEvent("UNIT_MAXHEALTH");
	self:UnregisterEvent("PLAYER_LEVEL_UP");
	self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	self:UnregisterEvent("PLAYER_TALENT_UPDATE");
	self.Frame:Hide();
end


--[[ OPTIONS ]]--
local function CreateLocalization()
	local L = LibStub("AceLocale-3.0"):GetLocale(MAJOR, true);
	if L then
		return L;
	end

	locale = "enUS";
	--[[ --------------------- ENGLISH LOCALE STRINGS --------------------- ]]--
	L = LibStub("AceLocale-3.0"):NewLocale(MAJOR, locale, true);
	L.DESCRIPTION = " ";
	L.SHOW_ALWAYS = "Always show bar";
	L.SHOW_ALWAYS_DESC = "If checked, the bar will always be visible. Otherwise it will only be visible if you actually have at least one point vengeance.";
	L.BACKGROUND_COLOR = "Background color";
	L.BAR = "Bar";
	L.BAR_COLOR = "Bar color";
	L.BAR_WIDTH = "Width";
	L.BAR_HEIGHT = "Height";
	L.BAR_SHOWBORDER = "Show border";
	L.BORDER_WIDTH = "Border width";
	L.SAMPLE_VALUE = "Sample data";
	L.SAMPLE_VALUE_RESET = "Resets the bar so that sample data is not shown anymore.";
	L.FRAME = "Frame";
	L.FRAME_LOCK = "Lock bar";
	L.FRAME_LOCK_DESC = "Locks the bar (disables mouse interaction)";
	L.TEXTURE = "Texture";
	L.TEXT = "Text";
	L.TEXT_COLOR = "Text color";
	L.TRY = "";

	return LibStub("AceLocale-3.0"):GetLocale(MAJOR);
end

local optionstable = nil;
local function GetOptionsTable()
	if optionstable then
		return optionstable;
	end
	local L = CreateLocalization();
	local SM = LibStub("LibSharedMedia-3.0");
	local SV = VengeanceSavedVariable;
	local Frame = Addon.Frame;
	
	optionstable = {
		type = "group",
		name = MAJOR,
		childGroups = "tab",
		args = {
			desc = {
				name = L.DESCRIPTION,
				type = "description",
				order = 1,
			},
			visibility = {
				name = L.SHOW_ALWAYS,
				desc = L.SHOW_ALWAYS_DESC,
				order = 2,
				type = "toggle",
				set = function(info, val)
					SV.ShowAlways = val;
					if val then
						Frame:Show();
					else
						Addon:OnUpdate();
					end
				end,
				get = function(info) return SV.ShowAlways; end,
			},
			mouse = {
				name = L.FRAME_LOCK,
				desc = L.FRAME_LOCK_DESC,
				order = 3,
				width = "double",
				type = "toggle",
				set = function(info, val)
					SV.MouseEnabled = not val;
					Frame:EnableMouse(not val);
				end,
				get = function(info) return not SV.MouseEnabled; end,
			},
			try = {
				name = L.TRY,
				order = 4,
				type = "group",
				inline = true,
				args = {
					sample = {
						name = L.SAMPLE_VALUE,
						order = 1,
						type = "execute",
						func = function(info, val)
							Addon.Frame:SetValue(5);
							Addon.Frame:SetMinMaxValues(0,10);
						end,
					},
					samplereset = {
						name = RESET,
						desc = L.SAMPLE_VALUE_RESET,
						order = 2,
						type = "execute",
						width = "half",
						func = function(info, val)
							Addon:OnMaxHealthUpdate();
						end,
					},
				},
			},
			frame = {
				name = L.FRAME,
				order = 7,
				type = "group",
				inline = true,
				args = {
					width = {
						name = L.BAR_WIDTH,
						type = "range",
						order = 31,
						min = 0,
						max = 2000,
						softMax = 500,
						step = 1,
						width = "double",
						set = function(info, val)
							Frame:SetWidth(val);
							SV.FrameRect = {Frame:GetRect()};
						end,
						get = function(info) return SV.FrameRect[3] or 200; end,
					},
					height = {
						name = L.BAR_HEIGHT,
						type = "range",
						order = 32,
						min = 0,
						max = 1000,
						softMax = 200,
						step = 1,
						width = "double",
						set = function(info, val)
							Frame:SetHeight(val);
							SV.FrameRect = {Frame:GetRect()};
						end,
						get = function(info) return SV.FrameRect[4] or 20; end,
					},
					opacity = {
						name = OPACITY,
						type = "range",
						order = 20,
						min = 0,
						max = 1,
						step = .01,
						bigStep = .1,
						set = function(info, val) SV.Opacity = val; Frame:SetAlpha(val); end,
						get = function(info) return SV.Opacity; end,
					},
					borderwidth = {
						name = L.BORDER_WIDTH,
						type = "range",
						order = 21,
						min = 0,
						max = 1000,
						softMax = 10,
						step = .01,
						bigStep = .1,
						set = function(info, val)
							Addon:SetBorderWidth(val);
						end,
						get = function(info) return SV.BorderWidth or 3; end,
					},
					border = {
						name = L.BAR_SHOWBORDER,
						type = "toggle",
						order = 22,
						set = function(info, val)
							SV.ShowBorder = val;
							Addon:SetBorderVisibility(val);
						end,
						get = function(info) return SV.ShowBorder; end,
					},
				},
			},
			bar = {
				name = L.BAR,
				order = 8,
				type = "group",
				inline = true,
				args = {
					bartexture = {
						name = L.TEXTURE,
						order = 1,
						type = "select",
						set = function(info, val)
							Addon:SetTexture(SM:List("statusbar")[val]);
						end,
						get = function(info)
							for k,v in pairs(SM:List("statusbar")) do
								if v == SV.Texture then
									return k;
								end
							end
						end,
						values = function(info) return SM:List("statusbar"); end,
					},
					barcolor = {
						name = L.BAR_COLOR,
						order = 3,
						type = "color",
						hasAlpha = true,
						set = function(info, r,g,b,a)
							SV.ColorBar = {r,g,b,a};
							Addon:SetColorBar(r,g,b,a);
						end,
						get = function(info) return unpack(SV.ColorBar or {1,1,1,1}); end,
					},
					backgroundcolor = {
						name = L.BACKGROUND_COLOR,
						order = 4,
						type = "color",
						hasAlpha = true,
						set = function(info, r,g,b,a)
							SV.ColorBackground = {r,g,b,a};
							Addon:SetColorBackground(r,g,b,a);
						end,
						get = function(info) return unpack(SV.ColorBackground or {0,0,0,.5}); end,
					},
				},
			},
			text = {
				name = L.TEXT,
				order = 8,
				type = "group",
				inline = true,
				args = {
					--[[bartexture = {
						name = L.TEXTURE,
						order = 1,
						type = "select",
						set = function(info, val)
							Addon:SetTexture(SM:List("statusbar")[val]);
						end,
						get = function(info)
							for k,v in pairs(SM:List("statusbar")) do
								if v == SV.Texture then
									return k;
								end
							end
						end,
						values = function(info) return SM:List("statusbar"); end,
					},]]
					textcolor = {
						name = L.TEXT_COLOR,
						order = 3,
						type = "color",
						hasAlpha = true,
						set = function(info, r,g,b,a)
							SV.ColorText = {r,g,b,a};
							Addon:SetColorText(r,g,b,a);
						end,
						get = function(info) return unpack(SV.ColorText or {1,1,1,1}); end,
					},
				},
			},
		},
	};
	
	return optionstable;
end


local AceConfig = LibStub("AceConfig-3.0");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");

AceConfig:RegisterOptionsTable(MAJOR, GetOptionsTable);
optionstablecategory = AceConfigDialog:AddToBlizOptions(MAJOR, nil, nil);
